/*
  This data visualization shows the check-out
  frequency map of three different versions of Blade Runner
  from 2005 to 2015.
  
  Author: Jieliang Luo
  Supervisor: George Legrady
  Date: Jan 4th, 2016
  Modified on: Jan 25th, 2017
*/

/*
Query:
SELECT 
    title,
    bibNumber,
    DATE(checkOut) AS checkOut,
    DATE(checkIn),
    TIMESTAMPDIFF(DAY, checkOut, checkIn),
    itemNumber
FROM
    spl3._rawXmlDataCheckIns
WHERE
    title = 'Blade Runner' AND checkIn > 0
        AND YEAR(checkOut) > 1970
GROUP BY checkOut
ORDER BY itemNumber , checkOut
*/

final int TOTAL_DAYS = 4015; //total of 11 years
final int startYear = 2005;
final int TIMELINE_LENGTH = 1200;

Table table, itemTable;
int numRows, numColumns;
int itemRows; //the total amount of item numbers 
float maxValue;

int[] bibNumber = {2132112, 1943806, 2479267};
color[] colors = {#36b11a, #1b037f, #9d0387};

ArrayList<ArrayList<String>> itemRecords = new ArrayList<ArrayList<String>>();
ArrayList<ArrayList<String>> bibRecords = new ArrayList<ArrayList<String>>();
int[] itemNumber;

void setup(){
 size(1400, 800);
 
 table = loadTable("blade_runner_itemNumber.csv", "header");
 numRows = table.getRowCount(); 
 
 //loading item number information
 itemTable = loadTable("itemNumbers.csv", "header");
 itemRows = itemTable.getRowCount();
 
 //Writing item numbers to itemNumber array
 itemNumber = new int[itemRows];
 for(int i=0; i<itemRows; i++){
   itemNumber[i] = itemTable.getInt(i,1);
   itemRecords.add(new ArrayList<String>());
   bibRecords.add(new ArrayList<String>());
 }
 
 for(int i=0; i<numRows; i++){
   for(int j=0; j<itemNumber.length; j++){
    if(table.getInt(i,5) == itemNumber[j]){
      itemRecords.get(j).add(table.getString(i,2));
      bibRecords.get(j).add(table.getString(i,1));
      break;
    }
   }
 }
 
 println("Setup Done!");

}

void draw(){
 background(255);
  
  //Draw inner rect background
 noStroke();
 fill(200);
 rect((width-TIMELINE_LENGTH)/2, 100, TIMELINE_LENGTH, 650); 
  
 //Draw Horizontal Labels    
 strokeWeight(5);
 for(int i=0; i<11; i++){
  textSize(12);
  fill(0);
  textAlign(CENTER);
  text(2005+i, (width-TIMELINE_LENGTH)/2 + TIMELINE_LENGTH*i/10, height-35);
  
  stroke(255);
  strokeWeight(1);
  line((width-TIMELINE_LENGTH)/2 + TIMELINE_LENGTH*i/10, height-35, (width-TIMELINE_LENGTH)/2 + TIMELINE_LENGTH*i/10, 100);
 }
 
 //Draw Vertial Labels
 for(int i=0; i<itemRecords.size(); i++){
   textSize(9);
   textAlign(RIGHT, CENTER);
   text(itemNumber[i], (width-TIMELINE_LENGTH)/2, i*10+120);
 }
 
 //Draw Color Labels
 for(int i=0; i<3; i++){
   textAlign(CENTER, TOP);
   fill(0);
   text(bibNumber[i], width*3/4+TIMELINE_LENGTH/4, 100 + i*50);
   fill(colors[i]);
   rect(width*3/4+TIMELINE_LENGTH/4-textWidth(str(bibNumber[i]))/2, 110+i*50, textWidth(str(bibNumber[i])), 15);
 }
 
 //Visualize Data
 strokeWeight(1);
 for(int i=0; i<itemRecords.size(); i++){
   for(int j=0; j<itemRecords.get(i).size(); j++){
     if(int(bibRecords.get(i).get(j)) == bibNumber[0]) stroke(colors[0]);
     else if(int(bibRecords.get(i).get(j)) == bibNumber[1]) stroke(colors[1]);
     else if(int(bibRecords.get(i).get(j)) == bibNumber[2]) stroke(colors[2]);
     
     float startPoint = (width-TIMELINE_LENGTH)/2 + caculateDays(itemRecords.get(i).get(j))*TIMELINE_LENGTH/TOTAL_DAYS;
     rect(startPoint, i*10+120, 1, 5);
   }
 }
 
 //Draw Title
 textAlign(CENTER);
 textSize(18);
 text("Check-Out Frequency Map of Blade Runner from 2005 to 2015", width/2, 50);
}

int caculateDays(String date){
 String[] dates = split(date, '-');
 int Year = int(dates[0]);
 int Month = int(dates[1]);
 int Day = int(dates[2]);
 int days = (Year-startYear)*365 + Month*30 + Day;
 return days;
}